// FavoriteApp.cpp
// (C) Jonas Sundström @ Kirilla.com

#define DEBUG 0
#include <Debug.h>

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>

#include <Application.h>
#include <Roster.h>
#include <Alert.h>
#include <String.h>
#include <Path.h>
#include <Entry.h>
#include <Directory.h>
#include <FindDirectory.h>
#include <Beep.h>
#include <PopUpMenu.h>
#include <Volume.h>
#include <VolumeRoster.h>
#include <Messenger.h>
#include <Node.h>
#include <fs_attr.h>
#include <SupportDefs.h>
#include <Mime.h>

#include "Defs.h"
#include "FavoriteApp.h"
#include "URL_file.h"

FavoriteApp::FavoriteApp	(void)
 : BApplication	(SPRINGBOARD_APP_SIG)
{
	// set up defaults
	m_mime					=	new	BMimeType();
	m_window				=	NULL;
	m_got_refs				=	false;
	m_got_argv				=	false;
	m_do_handle_favorites	=	true;
	m_do_handle_bookmarks	=	false;
	m_winframe.Set(300,200,495,320);
	
	file_app	=	NETPOSITIVE_APP_SIG;
	ftp_app		=	NETPOSITIVE_APP_SIG;
	http_app	=	NETPOSITIVE_APP_SIG;
	https_app	=	NETPOSITIVE_APP_SIG;
	rtsp_app	=	NETPOSITIVE_APP_SIG;
	telnet_app	=	NETPOSITIVE_APP_SIG;
	mailto_app	=	NETPOSITIVE_APP_SIG;
	
	// create settings folder
	MakeSettingsFolder();
	
	// register filetype + set as pref app	      (if it does not already exist)
	RegisterFavorite();	
	RegisterBookmark();	
	RemoveOldFavorite();
		
	// read settings
	ReadSettings();
	
	// do we handle Favorites and Bookmarks ?
	get_fav_pref_apps();
	get_book_pref_apps();
	
	// create window
	m_window = new FavoriteWindow(m_winframe);

	// set up window 
	set_win_pref_apps();	//snooze(1000);
	get_URL_apps();			//snooze(1000);
	get_URL_pref_apps();
	
	// start looking for updates to URL filetypes
	m_mime->StartWatching(be_app_messenger);	

}

FavoriteApp::~FavoriteApp	(void)	
{
	m_mime->StopWatching(be_app_messenger);
}

void
FavoriteApp::ReadyToRun	(void)
{
	if (m_got_refs)
	{
		// wait for some more refs.. 
		m_got_refs = false;
		
		BMessage msg	(SPRINGBOARD_QUIT_NOW);
		m_msg_runner =	new BMessageRunner(this, & msg, 1000000);	// 2 secs
	}
	else if (m_got_argv)
	{
		PostMessage(B_QUIT_REQUESTED);
	}
	else
		m_window->Show();
}

void 
FavoriteApp::MessageReceived	(BMessage * a_message)
{
	PRINT(("FavoriteApp::MessageReceived()\n"));

	switch(a_message->what)
	{
		case SPRINGBOARD_QUIT_NOW:	// wait, then quit -- for refs that come in late
				if (m_got_refs == false)
					be_app->PostMessage(B_QUIT_REQUESTED);
				else
				{
					m_got_refs = false;
					delete		m_msg_runner;
					BMessage 	msg	(SPRINGBOARD_QUIT_NOW);
					m_msg_runner =	new BMessageRunner(this, & msg, 1000000);	// wait 2 more secs
				}
				break;
				
		case SPRINGBOARD_SAVE_SETTINGS:

				SaveSettings(a_message);
				break;

		case SPRINGBOARD_HANDLE_FAVORITES:

				onoff_fav_pref_app();
				break;	

		case SPRINGBOARD_HANDLE_BOOKMARKS:

				onoff_book_pref_app();
				break;	

		case SPRINGBOARD_FILE_DROPPED:

				DropReceived	(a_message);
				break;	

		case SPRINGBOARD_HELP_REQUESTED:				

				Help();
				break;

		case SPRINGBOARD_LAUNCH_URL_FROM_CLIPBOARD:				

				LaunchClipboard();
				break;
				
		case SPRINGBOARD_BOOKMARK_URL_FROM_CLIPBOARD:
				Clipboard2Bookmark();
				break; 
				
		case SPRINGBOARD_OPEN_BOOKMARK_FOLDER:
				system("/system/Tracker /boot/home/config/settings/NetPositive/Bookmarks &");
				break;

		case B_META_MIME_CHANGED:						// Filetype DB Update

				mime_changed	(a_message);
				break;

		case SPRINGBOARD_URL_APP_CHANGE:				// change of URL's pref app

				set_URL_pref_app	(a_message);			// (from FavoriteWindow)
				break;

		case SPRINGBOARD_REPLICANT_QUITS:

				m_window->Lock();
				m_window->PostMessage(SPRINGBOARD_REPLICANT_QUITS);
				m_window->Unlock();
				break;

		default:
				BApplication::MessageReceived(a_message);
				break;			
	}
}

void
FavoriteApp::AboutRequested	(void)
{
	BAlert * alert = new BAlert("About Springboard", 
			"This is Springboard 2.2\n"
			B_UTF8_COPYRIGHT " Jonas Sundström, Kirilla.com\n\n"
			"Neat, don't you think? :)\n",
			"Homepage", "Help", "Ok", B_WIDTH_AS_USUAL, B_IDEA_ALERT);

	int32 reply = alert->Go();
	
	if (reply == 0)	Launch	(SPRINGBOARD_HTTP_URL_TYPE, "http://www.kirilla.com/");
	if (reply == 1)	Help(); 	
}

void
FavoriteApp::ErrorMessage	(const char * a_text, int32 a_status)
{
	PRINT(("%s: %s\n", a_text, strerror(a_status)));
}

status_t 
FavoriteApp::MakeSettingsFolder	()
{
	status_t	status;
	BPath		path;
	if ((status = find_directory(B_USER_SETTINGS_DIRECTORY, & path)) != B_OK)
		return status;
	
	BEntry 		entry	(path.Path());

	// settings
	if (entry.Exists()	==	false	||	entry.IsDirectory()	==	false)	
		return B_ERROR;
	
	BDirectory	mother	(path.Path());
	BDirectory	baby;
	
	// Kirilla
	path.SetTo(path.Path(), "Kirilla");
	entry.SetTo(path.Path());
	if (! entry.Exists())
	{
		status	=	mother.CreateDirectory("Kirilla", & baby);
		if (status != B_OK && status != B_FILE_EXISTS)				return status;
	}
	else 
		if (! entry.IsDirectory())
			return B_FILE_EXISTS;
	
	if ((status = mother.SetTo(path.Path())) != B_OK)			return status;

	// Springboard
	path.SetTo(path.Path(), "Springboard");
	entry.SetTo(path.Path());
	if (! entry.Exists())
	{
		status	=	mother.CreateDirectory("Springboard", & baby);
		if (status != B_OK && status != B_FILE_EXISTS)				return status;
	}
	else 
		if (! entry.IsDirectory())
			return B_FILE_EXISTS;
	
	if ((status = mother.SetTo(path.Path())) != B_OK)			return status;

	entry.SetTo(path.Path());
	if (entry.Exists()	&&	entry.IsDirectory())	return B_OK;
	else											return B_ERROR;
}

void 
FavoriteApp::RefsReceived	(BMessage * a_message)
{
	m_got_refs 	= true;

	entry_ref	ref;
	int32		count	=	0;

	while(a_message->FindRef("refs", count++, &ref) == B_OK)
		ProcessRefs(& ref, SPRINGBOARD_OPEN);
}

void 
FavoriteApp::DropReceived	(BMessage * a_message)
{
	PRINT(("FavoriteApp::DropReceived()\n"));

	status_t	status	=	B_OK;
	entry_ref	ref;
	int32		count	=	0;

	while(a_message->FindRef("refs", count++, &ref) == B_OK)
	{
		status = ProcessRefs(& ref, SPRINGBOARD_CONVERT);
		
		if (status != B_OK)
		{
			BAlert * alert = new BAlert("Springboard Error", 
			"Conversion aborted.\n\n"
			"If you're trying to convert your Windows Favorites, "
			"copy them to a BeOS volume first.\n\n"
			"If you're trying convert files "
			"already on your Be volume, and the conversion fails, "
			"try with a lesser number of files.",
			
			"Ok", NULL, NULL, B_WIDTH_AS_USUAL, B_STOP_ALERT);
			alert->Go(NULL);

			break; // aborts batch of conversions
		}
	}
}

status_t 
FavoriteApp::ProcessRefs	(entry_ref * ref, bool do_convert)
{
	PRINT(("FavoriteApp::ProcessRefs()\n"));

	// resolve link
	BEntry entry(ref, true);
	entry.GetRef(ref);		
	
	if (entry.IsDirectory())
	{
		BDirectory	dir	(ref);
		BList	list (20);
		
		while	(dir.GetNextRef	(ref) == B_OK)
		{
			entry_ref * tmp	=	new entry_ref;
			*tmp = *ref;							// before I added the BList cache
			list.AddItem((void *) tmp);				// the refs would appear twice
		}											// (probably due to our conversion of files)
		
		while ((ref = (entry_ref *) list.RemoveItem((int32) 0)))
		{
			ProcessRefs	(ref, do_convert);
			delete ref;
		}
			
		return B_OK;
	}
	
	// init file object
	URL_file	file (ref);
	if (file.InitCheck() != B_OK)
	{
		PRINT(("Springboard: file.InitCheck() != B_OK: %s\n", ref->name));
		return B_OK; // skip file, but continue with batch
	}
	
	if	(do_convert)
	{
		status_t	status	=	B_OK;
		status	=	file.Convert();
		if (status != B_OK)
			return B_ERROR;  // aborts batch of conversions
	}
	else
		Launch(file.URL());
	
	return B_OK;
}

status_t 
FavoriteApp::SaveSettings(BMessage * a_message)
{
	status_t	status;
	BMessage 	settings;
	BPath		path;
	BFile		file;
	BRect		winframe;

	// add settings to message
	if (a_message->	FindRect("winframe", &winframe) == B_OK)
		settings.AddRect("winframe", winframe);
	
	// get path to settings-file
	if ((status = find_directory(B_USER_SETTINGS_DIRECTORY, & path)) != B_OK)
		return status;

	if ((status = path.SetTo(path.Path(),"Kirilla/Springboard/settings")) != B_OK)
		return status;
		
	if ((status	= file.SetTo(path.Path(), B_READ_WRITE | B_CREATE_FILE | B_ERASE_FILE)) != B_OK)
		return status;

	if ((status	= settings.Flatten(& file))	!=	B_OK)
		return status;

	return B_OK;
}

status_t
FavoriteApp::ReadSettings()
{
	status_t	status;
	BMessage	settings;
	BPath		path;
	BFile		file;
	BRect 		frame;

	// get path to settings-file
	if ((status = find_directory(B_USER_SETTINGS_DIRECTORY, & path)) != B_OK)
		return status;

	if ((status = path.SetTo(path.Path(),"Kirilla/Springboard/settings")) != B_OK)
		return status;
		
	if ((status	= file.SetTo(path.Path(), B_READ_ONLY)) != B_OK)
		return status;

	// get message from file
	if ((status	=	settings.Unflatten(& file)) != B_OK)
		return status;

	// get settings from message
	if (settings.FindRect("winframe", & frame) == B_OK)		m_winframe = frame;

	return B_OK;
}

void 
FavoriteApp::Launch	(const char * a_URL)
{
	PRINT(("Launch()\n"));

	// URL -> APP mapping
	BString URL	(a_URL);
	BString app;
	
	URL.ToLower();
	if (URL.FindFirst("file://") == 0)		app = file_app;
	if (URL.FindFirst("ftp://") == 0)		app = ftp_app;
	if (URL.FindFirst("http://") == 0)		app = http_app;
	if (URL.FindFirst("https://") == 0)		app = https_app;
	if (URL.FindFirst("rtsp://") == 0)		app = rtsp_app;
	if (URL.FindFirst("telnet://") == 0)	app = telnet_app;
	if (URL.FindFirst("mailto:") == 0)		app = mailto_app;
	
	// send URL to app 
	BMessage msg(B_ARGV_RECEIVED); 
	msg.AddString("argv", "app"); 
	msg.AddString("argv", a_URL);  
	msg.AddInt32("argc", 2); 
	
	PRINT(("file_app: %s\n", file_app.String()));
	PRINT(("ftp_app: %s\n", ftp_app.String()));
	PRINT(("http_app: %s\n", http_app.String()));
	PRINT(("https_app: %s\n", https_app.String()));
	PRINT(("rtsp_app: %s\n", rtsp_app.String()));
	PRINT(("telnet_app: %s\n", telnet_app.String()));
	PRINT(("mailto_app: %s\n", mailto_app.String()));
	
	PRINT(("app: %s, URL: %s\n", app.String(), a_URL));

	be_roster->Launch (app.String(), &msg ); 
}

void 
FavoriteApp::Launch	(const char * a_app_sig, const char * a_URL)
{
	BMessage msg(B_ARGV_RECEIVED); 
	msg.AddString("argv", "app"); 
	msg.AddString("argv", a_URL);  

	msg.AddInt32("argc", 2); 
	be_roster->Launch (a_app_sig, &msg ); 
}

void 
FavoriteApp::Help	(void)
{
	BAlert * alert = new BAlert("Springboard Help", 
			"Springboard is designed to stay out of your way. "
			"When you've set it up, your Favorites/Bookmarks "
			"will open in the applications of your choice, "
			"no questions asked.\n\n"
			
			"Use Springboard's Settings-menu, or your FileTypes preferences, "
			"to specify which application "
			"handles which kind of URL, and if you want "
			"Springboard to open both Favorites and Bookmarks.\n\n"
			
			"There's also a dropview, where you can drop Favorites you want to "
			"convert to Bookmarks, and vice versa.\n\n"
			
			"You will find that in spite of Springboard being the \"Preferred Application\" "
			"of the Bookmark filetype, your bookmarks will continue to open in NetPositive. "
			"This is because NetPositive needlessly sets the BEOS:PREF_APP attribute on every "
			"Bookmark it creates. You can however, easily remedy that, by dropping them -twice- "
			"in Springboards convert-view.",
			
			"Ok", NULL, NULL, B_WIDTH_AS_USUAL, B_IDEA_ALERT);

	alert->Go(NULL);
}

void  
FavoriteApp::get_URL_apps	(void)
{
	PRINT(("get_URL_apps()\n"));

	BMessage	msg;
	BMimeType	mime;

	mime.SetTo(SPRINGBOARD_FILE_URL_TYPE);
	mime.GetSupportingApps(& msg);
	msg.what = SPRINGBOARD_URL_APP_INIT;
	msg.AddString("URL_TYPE", SPRINGBOARD_FILE_URL_TYPE);
	m_window->PostMessage(& msg);
	
	mime.SetTo(SPRINGBOARD_FTP_URL_TYPE);
	mime.GetSupportingApps(& msg);
	msg.what = SPRINGBOARD_URL_APP_INIT;
	msg.AddString("URL_TYPE", SPRINGBOARD_FTP_URL_TYPE);
	m_window->PostMessage(& msg);
	
	mime.SetTo(SPRINGBOARD_HTTP_URL_TYPE);
	mime.GetSupportingApps(& msg);
	msg.what = SPRINGBOARD_URL_APP_INIT;
	msg.AddString("URL_TYPE", SPRINGBOARD_HTTP_URL_TYPE);
	m_window->PostMessage(& msg);
	
	mime.SetTo(SPRINGBOARD_HTTPS_URL_TYPE);
	mime.GetSupportingApps(& msg);
	msg.what = SPRINGBOARD_URL_APP_INIT;
	msg.AddString("URL_TYPE", SPRINGBOARD_HTTPS_URL_TYPE);
	m_window->PostMessage(& msg);
	
	mime.SetTo(SPRINGBOARD_RTSP_URL_TYPE);
	mime.GetSupportingApps(& msg);
	msg.what = SPRINGBOARD_URL_APP_INIT;
	msg.AddString("URL_TYPE", SPRINGBOARD_RTSP_URL_TYPE);
	m_window->PostMessage(& msg);
	
	mime.SetTo(SPRINGBOARD_TELNET_URL_TYPE);
	mime.GetSupportingApps(& msg);
	msg.what = SPRINGBOARD_URL_APP_INIT;
	msg.AddString("URL_TYPE", SPRINGBOARD_TELNET_URL_TYPE);
	m_window->PostMessage(& msg);
	
	mime.SetTo(SPRINGBOARD_MAILTO_URL_TYPE);
	mime.GetSupportingApps(& msg);
	msg.what = SPRINGBOARD_URL_APP_INIT;
	msg.AddString("URL_TYPE", SPRINGBOARD_MAILTO_URL_TYPE);
	m_window->PostMessage(& msg);
}

void  
FavoriteApp::get_URL_pref_apps	(void)
{
	PRINT(("get_URL_pref_apps()\n"));

	BMessage	msg	(SPRINGBOARD_URL_PREF_APP_INIT);
	BMimeType	mime;
	char	*	pref 	=	new char [B_MIME_TYPE_LENGTH];
	BString		app_string;
	
	mime.SetTo(SPRINGBOARD_FILE_URL_TYPE);
	mime.GetPreferredApp(pref);
	msg.AddString(SPRINGBOARD_FILE_URL_TYPE, pref);
	file_app = pref;
	
	mime.SetTo(SPRINGBOARD_FTP_URL_TYPE);
	mime.GetPreferredApp(pref);
	msg.AddString(SPRINGBOARD_FTP_URL_TYPE, pref);
	ftp_app = pref;
	
	mime.SetTo(SPRINGBOARD_HTTP_URL_TYPE);
	mime.GetPreferredApp(pref);
	msg.AddString(SPRINGBOARD_HTTP_URL_TYPE, pref);
	http_app = pref;
	
	mime.SetTo(SPRINGBOARD_HTTPS_URL_TYPE);
	mime.GetPreferredApp(pref);
	msg.AddString(SPRINGBOARD_HTTPS_URL_TYPE, pref);
	https_app = pref;
	
	mime.SetTo(SPRINGBOARD_RTSP_URL_TYPE);
	mime.GetPreferredApp(pref);
	msg.AddString(SPRINGBOARD_RTSP_URL_TYPE, pref);
	rtsp_app = pref;
	
	mime.SetTo(SPRINGBOARD_TELNET_URL_TYPE);
	mime.GetPreferredApp(pref);
	msg.AddString(SPRINGBOARD_TELNET_URL_TYPE, pref);
	telnet_app = pref;
	
	mime.SetTo(SPRINGBOARD_MAILTO_URL_TYPE);
	mime.GetPreferredApp(pref);
	msg.AddString(SPRINGBOARD_MAILTO_URL_TYPE, pref);
	mailto_app = pref;
	
	m_window->PostMessage(& msg);
	delete pref;
}

void 
FavoriteApp::RegisterFavorite	(void)
{
	status_t	status	=	B_OK;
	char * buffer	=	new char [B_MIME_TYPE_LENGTH];
	buffer[B_MIME_TYPE_LENGTH - 1]	=	'\0';

	BMimeType	mime;
	mime.SetTo(SPRINGBOARD_FAVORITE_TYPE);
	
	// install mimetype
	if (! mime.IsInstalled())				
	{
		mime.Install();
		
		// For some reason this code does not appear to run, ever.
		// Does the BeOS auto-install the apps supported type(s)
		// and set the preferred app on them?

		PRINT(("mime.Install();\n"));
	}
	
	// filename extension
	BMessage	ext_msg;

	status = mime.GetFileExtensions(& ext_msg);
	
	type_code	type	=	0;
	int32		count	=	0;
	ext_msg.GetInfo("extensions", & type, & count);	 // how many extensions does the filetype specify
	
	if ((status != B_OK) || count < 2) 							// it's our filetype, so we're not being
	{															// a bad citizen by making sure that 
																// these two filename extensions
		ext_msg.MakeEmpty();									// are coupled with the filetype.
		ext_msg.AddString("extensions", "url");					// Unless of course, these extensions are 
		ext_msg.AddString("extensions", "URL");					// used by some other, prominent filetype
		mime.SetFileExtensions(& ext_msg);						// 
																// if we were really anal about this
		PRINT(("mime.SetFileExtensions(& ext_msg);\n"));		// we'd check the exact extensions
	}															// and not just them being 2
	
	// short description
	status = mime.GetShortDescription(buffer);
	BString string	(buffer);
	
	if ((status != B_OK))
	{
		mime.SetShortDescription("Favorite");
		
		PRINT(("mime.SetShortDescription(Favorite);\n"));
	}
	
	// long description
	status = mime.GetLongDescription(buffer);
	if ((status != B_OK))
	{
		mime.SetLongDescription("Internet Explorer Favorite");

		PRINT(("mime.SetLongDescription(Internet Explorer Favorite);\n"));
	}
	
	// preferred application
	status = mime.GetPreferredApp(buffer);
	if (status != B_OK)											// the most correct thing would be to check	for 
	{															// B_ENTRY_NOT_FOUND, but we catch that and any
		mime.SetPreferredApp(SPRINGBOARD_APP_SIG);				// other failure by checking for (status != B_OK)
																// This is no life-support machine so we can afford
		PRINT(("mime.SetPreferredApp(SPRINGBOARD_APP_SIG);\n"));	// to crash & burn once every blue moon
	
		// For some reason this code does not appear
		// to be run the first time the app runs 
		// Does the BeOS auto-install the apps supported type(s)
		// and set the preferred app on them?
	}

	delete [] buffer;
}

void 
FavoriteApp::RegisterBookmark	(void)		// see comments of FavoriteApp::RegisterFavorite	(void)
{
	status_t	status	=	B_OK;
	char * buffer	=	new char [B_MIME_TYPE_LENGTH];
	buffer[B_MIME_TYPE_LENGTH - 1]	=	'\0';

	BMimeType	mime;
	mime.SetTo(SPRINGBOARD_BOOKMARK_TYPE);
	
	// install mimetype
	if (! mime.IsInstalled())				
		mime.Install();
	
	// short description
	status = mime.GetShortDescription(buffer);
	if ((status != B_OK))
		mime.SetShortDescription("Bookmark");
	
	// long description
	status = mime.GetLongDescription(buffer);
	if ((status != B_OK))
		mime.SetLongDescription("Bookmark for a web page.");
	
	// preferred application
	status = mime.GetPreferredApp(buffer);
	if (status != B_OK)
		mime.SetPreferredApp(SPRINGBOARD_APP_SIG);

	delete [] buffer;
}

void 
FavoriteApp::RemoveOldFavorite	(void)
{
	PRINT(("RemoveOldFavorite()\n"));

	BMimeType	mime;
	mime.SetTo(SPRINGBOARD_OLD_FAVORITE_TYPE);
	mime.Delete();
}

void
FavoriteApp::set_URL_pref_app	(BMessage * a_message)	// called by URL menutiems
{
	PRINT(("set_URL_pref_app()\n"));
	
	BString		url;
	BString		app;

	a_message->	FindString("URL_TYPE", & url);
	a_message->	FindString("APP_SIG", & app);
	
	url.ToLower();
	app.ToLower();

	if (url == SPRINGBOARD_FILE_URL_TYPE)
		file_app	=	app;
	
	if (url == SPRINGBOARD_FTP_URL_TYPE)
		ftp_app		=	app;
	
	if (url == SPRINGBOARD_HTTP_URL_TYPE)
		http_app	=	app;
	
	if (url == SPRINGBOARD_HTTPS_URL_TYPE)
		https_app	=	app;
	
	if (url == SPRINGBOARD_RTSP_URL_TYPE)
		rtsp_app	=	app;
	
	if (url == SPRINGBOARD_TELNET_URL_TYPE)
		telnet_app	=	app;
		
	if (url == SPRINGBOARD_MAILTO_URL_TYPE)
		mailto_app	=	app;
	
	// add changes (if any) back to URL mime type
	set_pref_app	(url.String(), app.String());
}

void  
FavoriteApp::get_fav_pref_apps	(void)
{
	PRINT(("get_fav_pref_apps()\n"));
	
	BMimeType	mime;
	char	*	buffer 	=	new char [B_MIME_TYPE_LENGTH];
	BString		app_string;

	mime.SetTo(SPRINGBOARD_FAVORITE_TYPE);
	mime.GetPreferredApp(buffer);
	app_string	=	buffer;
	
	if (app_string == SPRINGBOARD_APP_SIG)
		m_do_handle_favorites				=	true;
	else
		m_do_handle_favorites				=	false;

	delete buffer;
}

void  
FavoriteApp::get_book_pref_apps	(void)
{
	PRINT(("get_book_pref_apps()\n"));

	BMimeType	mime;
	char	*	buffer 	=	new char [B_MIME_TYPE_LENGTH];
	BString		app_string;

	mime.SetTo(SPRINGBOARD_BOOKMARK_TYPE);
	mime.GetPreferredApp(buffer);
	app_string	=	buffer;
	
	if (app_string == SPRINGBOARD_APP_SIG)
		m_do_handle_bookmarks				=	true;
	else
		m_do_handle_bookmarks				=	false;

	delete buffer;
}

void
FavoriteApp::set_pref_app	(const char * a_mime_str,
									 const char * a_app_str)
{
	PRINT(("set_pref_app()\n"));

	BString	current_pref_app;
	char * buffer = new char [B_MIME_TYPE_LENGTH];
	
	BMimeType mime	(a_mime_str);

	if (! mime.IsInstalled())			// install if not exists
		mime.Install();
	
	mime.GetPreferredApp	(buffer);
	current_pref_app	=	buffer;
	
	if (current_pref_app != a_app_str)	// to avoid B_META_MIME_CHANGED
		mime.SetPreferredApp(a_app_str);
}

void
FavoriteApp::onoff_fav_pref_app	(void)
{
	PRINT(("onoff_fav_pref_app()\n"));
	
	m_do_handle_favorites	=	!	m_do_handle_favorites;	// invert bool

	if (m_do_handle_favorites)
	{
		set_pref_app	(SPRINGBOARD_FAVORITE_TYPE, SPRINGBOARD_APP_SIG);
		m_window->	PostMessage(SPRINGBOARD_HANDLE_FAVORITES_ON);
	}
	else
	{
		set_pref_app	(SPRINGBOARD_FAVORITE_TYPE, NULL);	// remove pref app
		m_window->	PostMessage(SPRINGBOARD_HANDLE_FAVORITES_OFF);
	}
}

void
FavoriteApp::onoff_book_pref_app	(void)
{
	PRINT(("onoff_book_pref_app()\n"));

	m_do_handle_bookmarks	=	!	m_do_handle_bookmarks;	// invert bool
	
	if (m_do_handle_bookmarks)
	{
		set_pref_app	(SPRINGBOARD_BOOKMARK_TYPE, SPRINGBOARD_APP_SIG);
		m_window->	PostMessage(SPRINGBOARD_HANDLE_BOOKMARKS_ON);
	}
	else
	{
		set_pref_app	(SPRINGBOARD_BOOKMARK_TYPE, NETPOSITIVE_APP_SIG); // default pref app
		m_window->	PostMessage(SPRINGBOARD_HANDLE_BOOKMARKS_OFF);
	}
	
}

void
FavoriteApp::mime_changed	(BMessage * a_message)
{
	PRINT(("mime_changed()\n"));
	
	int32		which;
	BString		type;
	BString		extra_type;;

	a_message->	FindInt32("be:which", & which);
	a_message->	FindString("be:type", & type);
	a_message->	FindString("be:extra_type", & extra_type);

	type.ToLower();

	if (which & B_PREFERRED_APP_CHANGED)
	{
		if (type == SPRINGBOARD_FAVORITE_TYPE)
		{
			get_fav_pref_apps();
			set_win_pref_apps();
		}
		if (type == SPRINGBOARD_BOOKMARK_TYPE)
		{
			get_book_pref_apps();
			set_win_pref_apps();
		}
		
		if	(
				type == SPRINGBOARD_FILE_URL_TYPE	||
				type == SPRINGBOARD_FTP_URL_TYPE	||
				type == SPRINGBOARD_HTTP_URL_TYPE	||
				type == SPRINGBOARD_HTTPS_URL_TYPE	||
				type == SPRINGBOARD_RTSP_URL_TYPE	||
				type == SPRINGBOARD_TELNET_URL_TYPE	||
				type == SPRINGBOARD_MAILTO_URL_TYPE
			)
		{
			get_URL_apps();
			get_URL_pref_apps();
		}
	}
}

void  
FavoriteApp::set_win_pref_apps	(void)
{
	PRINT(("set_win_pref_apps()\n"));

	if (m_do_handle_favorites)
		m_window->	PostMessage(SPRINGBOARD_HANDLE_FAVORITES_ON);
	else
		m_window->	PostMessage(SPRINGBOARD_HANDLE_FAVORITES_OFF);

	if (m_do_handle_bookmarks)
		m_window->	PostMessage(SPRINGBOARD_HANDLE_BOOKMARKS_ON);
	else
		m_window->	PostMessage(SPRINGBOARD_HANDLE_BOOKMARKS_OFF);
}

void 
FavoriteApp::ArgvReceived	(int32 argc, char **argv)
{
	m_got_argv	=	true;

	BString argument = argv[1];
	
	if (argument == "--clipboard")
		LaunchClipboard();
	else if (argument == "--clip2bookmark")
		Clipboard2Bookmark();
	else
	{
		for (int16 i = 1; i < argc; i++)
			Launch(argv[i]);
	}
}

void
FavoriteApp::LaunchClipboard	()
{
	BMessage	*	clip;
	if (be_clipboard->Lock())
	{
		clip = be_clipboard->Data();
	
		const char *	text;
		int32 			text_len;
		BString			url_string;
			
		clip->	FindData("text/plain", B_MIME_TYPE, (const void **)&text, &text_len);
			
		if (text != NULL)
			url_string.SetTo(text, text_len);
			
		Launch(url_string.String());	// FavoriteApp::Launch()
	
		be_clipboard->Unlock();
	}
	else
	{
		PRINT(("Could not lock clipboard.\n"));
		return;
	}
}

void
FavoriteApp::Clipboard2Bookmark	()
{
	BString			url_string;
	
	// get URL from clipboard
	BMessage	*	clip;
	if (be_clipboard->Lock())
	{
		clip = be_clipboard->Data();
	
		const char *	text;
		int32 			text_len;
		
			
		clip->	FindData("text/plain", B_MIME_TYPE, (const void **)&text, &text_len);
			
		if (text != NULL)
			url_string.SetTo(text, text_len);
			
		be_clipboard->Unlock();
	}
	else
	{
		PRINT(("Could not lock clipboard.\n"));
		return;
	}
	
	// save as Bookmark file
	
	//***********************************************************
	
	BEntry	entry	("/boot/home/config/settings/NetPositive/Bookmarks");
	if (! entry.Exists())
		return;
	
	BDirectory	bookmark_dir	(& entry);
	
	// create file
	BString file_name	=	"Bookmark";
	int32	counter	=	1;
	while(bookmark_dir.Contains(file_name.String()))
	{
		counter	++;
		file_name = "Bookmark";
		file_name.Append(" ");
		file_name << counter;
	}

	BFile file;
	bookmark_dir.CreateFile(file_name.String(), & file);
	
	file.WriteAttr("BEOS:TYPE", B_MIME_TYPE, 0, "application/x-vnd.Be-bookmark", 30);
	file.WriteAttr("META:url", B_STRING_TYPE, 0, url_string.String(), url_string.Length()+1);
	file.WriteAttr("META:title", B_STRING_TYPE, 0, url_string.String(), url_string.Length()+1);
	
	
	//***********************************************************
	
}

