//// FavoriteWindow.cpp
//

#define DEBUG 0
#include <Debug.h>

#include <stdio.h>
#include <stdlib.h>

#include <Application.h>
#include <Roster.h>
#include <InterfaceKit.h>
#include <String.h>
#include <FindDirectory.h>
#include <Path.h>
#include <File.h>
#include <Beep.h>
#include <Deskbar.h>

#include "Defs.h"
#include "FavoriteWindow.h"
#include "DropView.h"
#include "IconMenu.h"

FavoriteWindow::FavoriteWindow(BRect a_winframe)
: BWindow(BRect(300,200,495,320), "Springboard", B_TITLED_WINDOW, 0 ) //bottom: 275  // 435
{
	PRINT(("FavoriteWindow()\n"));
	
//	UI
	m_menubar		= new BMenuBar(BRect(0,0,195,20), "menubar");
	m_appmenu		= new BMenu("App");
	m_settingsmenu	= new BMenu("Settings");
	
	m_iconmenu		= new IconMenu(m_appmenu);
	
	m_menubar->		AddItem(m_iconmenu);
	m_menubar->		AddItem(m_settingsmenu);
	
	m_clipboarditem	= new BMenuItem("Launch URL from Clipboard" B_UTF8_ELLIPSIS, new BMessage(SPRINGBOARD_LAUNCH_URL_FROM_CLIPBOARD), 'C');
	m_clip2bookitem	= new BMenuItem("Bookmark URL from Clipboard", 	new BMessage(SPRINGBOARD_BOOKMARK_URL_FROM_CLIPBOARD), 'B');
	m_openbookitem	= new BMenuItem("Open Bookmark Folder" B_UTF8_ELLIPSIS, new BMessage(SPRINGBOARD_OPEN_BOOKMARK_FOLDER), 'O');
	
	m_aboutitem 	= new BMenuItem("About" B_UTF8_ELLIPSIS, new BMessage(B_ABOUT_REQUESTED));
	m_help_item		= new BMenuItem("Help" B_UTF8_ELLIPSIS, new BMessage(SPRINGBOARD_HELP_REQUESTED));
	m_quititem 		= new BMenuItem("Quit", new BMessage(B_QUIT_REQUESTED), 'Q');

	m_appmenu->		AddItem(m_clipboarditem);
	m_appmenu->		AddItem(m_clip2bookitem);
	m_appmenu->		AddItem(m_openbookitem);
	m_appmenu->		AddSeparatorItem();
	m_appmenu->		AddItem(m_aboutitem);
	m_appmenu->		AddItem(m_help_item);
	m_appmenu->		AddSeparatorItem();
	m_appmenu->		AddItem(m_quititem);

	m_handle_fav_item		= new BMenuItem("Handle Favorites", new BMessage(SPRINGBOARD_HANDLE_FAVORITES));
	m_handle_book_item		= new BMenuItem("Handle Bookmarks", new BMessage(SPRINGBOARD_HANDLE_BOOKMARKS));
	m_handle_fav_item->		SetTarget(be_app);
	m_handle_book_item->	SetTarget(be_app);
	
	m_file_menu			= new BMenu("file://");
	m_ftp_menu			= new BMenu("ftp://");
	m_http_menu			= new BMenu("http://");
	m_https_menu		= new BMenu("https://");
	m_rtsp_menu			= new BMenu("rtsp://");
	m_telnet_menu		= new BMenu("telnet://");
	m_mailto_menu		= new BMenu("mailto:");
	
	m_file_menu		->	SetRadioMode(true);
	m_ftp_menu		->	SetRadioMode(true);
	m_http_menu		->	SetRadioMode(true);
	m_https_menu	->	SetRadioMode(true);
	m_rtsp_menu		->	SetRadioMode(true);
	m_telnet_menu	->	SetRadioMode(true);
	m_mailto_menu	->	SetRadioMode(true);
	
	m_replicant_item	= new BMenuItem("Deskbar Icon", new BMessage(SPRINGBOARD_DESKBAR_ICON));

	m_settingsmenu->	AddItem(m_handle_fav_item);
	m_settingsmenu->	AddItem(m_handle_book_item);
	m_settingsmenu->	AddSeparatorItem();
	m_settingsmenu->	AddItem(m_file_menu);
	m_settingsmenu->	AddItem(m_ftp_menu);
	m_settingsmenu->	AddItem(m_http_menu);
	m_settingsmenu->	AddItem(m_https_menu);
	m_settingsmenu->	AddItem(m_rtsp_menu);
	m_settingsmenu->	AddItem(m_telnet_menu);
	m_settingsmenu->	AddItem(m_mailto_menu);
	m_settingsmenu->	AddSeparatorItem();
	m_settingsmenu->	AddItem(m_replicant_item);	

	float width, height;
	m_menubar->	GetPreferredSize(& width, & height);
	ResizeBy(0,height+2);
	
	topbox	=	new BBox(BRect(1,height+2,Bounds().right,Bounds().bottom), "view", B_FOLLOW_ALL_SIDES, B_NAVIGABLE);

	topbox	->AddChild(new	DropBox());

	AddChild(m_menubar);
	AddChild(topbox);

//	Init	
	MoveTo(a_winframe.LeftTop());
	ResizeTo(a_winframe.Width(), a_winframe.Height());
	SetSizeLimits(160, 15000, 100, 15000);
	
	BDeskbar deskbar;
	if (deskbar.HasItem(SPRINGBOARD_REPLICANT))
		m_replicant_item->SetMarked(true);
}

FavoriteWindow::~FavoriteWindow()	
{

}

void 
FavoriteWindow::MessageReceived		(BMessage * a_message)
{
	PRINT(("FavoriteWindow::MessageReceived()\n"));
	
	switch(a_message->what)
	{
		case SPRINGBOARD_URL_APP_INIT:				set_URL_apps	(a_message);			break;	
		case SPRINGBOARD_URL_PREF_APP_INIT:			set_URL_pref_apps	(a_message);		break;	
					
		case SPRINGBOARD_HANDLE_FAVORITES_ON:		m_handle_fav_item->SetMarked(true);		break;
		case SPRINGBOARD_HANDLE_FAVORITES_OFF:		m_handle_fav_item->SetMarked(false);	break;
		case SPRINGBOARD_HANDLE_BOOKMARKS_ON:		m_handle_book_item->SetMarked(true);	break;
		case SPRINGBOARD_HANDLE_BOOKMARKS_OFF:		m_handle_book_item->SetMarked(false);	break;
		
		case SPRINGBOARD_DESKBAR_ICON:				DeskbarIconOnOff();						break;
		case SPRINGBOARD_REPLICANT_QUITS:			ReplicantQuits();						break;
		
		case SPRINGBOARD_LAUNCH_URL_FROM_CLIPBOARD:		be_app->PostMessage(a_message);		break; 
		case SPRINGBOARD_BOOKMARK_URL_FROM_CLIPBOARD:	be_app->PostMessage(a_message);		break; 
		case SPRINGBOARD_OPEN_BOOKMARK_FOLDER:			be_app->PostMessage(a_message);		break; 
		
		case SPRINGBOARD_HELP_REQUESTED:			be_app->PostMessage(a_message);			break; 
		case B_ABOUT_REQUESTED:						be_app->PostMessage(a_message);			break;
		case SPRINGBOARD_FILE_DROPPED:				be_app->PostMessage(a_message);			break;

		default:	BWindow::MessageReceived(a_message);	break;			
	}
}

void
FavoriteWindow::DeskbarIconOnOff	(void)
{
	m_replicant_item->SetMarked(! m_replicant_item->IsMarked());

	BDeskbar deskbar;

	if (! deskbar.HasItem(SPRINGBOARD_REPLICANT))
	{
		entry_ref	ref;
		be_roster->FindApp(SPRINGBOARD_APP_SIG, & ref);
		deskbar.AddItem(& ref);
	}
	else
		deskbar.RemoveItem(SPRINGBOARD_REPLICANT);
}

void 
FavoriteWindow::ReplicantQuits	(void)
{
	m_replicant_item->SetMarked(false);
}

bool
FavoriteWindow::QuitRequested	(void)
{
	Save();
	be_app->PostMessage(B_QUIT_REQUESTED);
	return true;
}

void
FavoriteWindow::Save	(void)
{
	BMessage msg(SPRINGBOARD_SAVE_SETTINGS);
	msg.AddRect("winframe", Frame());
	
	be_app->PostMessage(& msg);
}

void
FavoriteWindow::set_URL_apps	(BMessage * a_message)
{
	PRINT(("FavoriteWindow::set_URL_apps()\n"));
	
	BString			url;
	BString			app;
	BMenu		*	menu		=	NULL;
	BMenuItem	*	item		=	NULL;
	
	int32	subs, count	=	0;
	
	a_message->	FindInt32("be:sub", & subs);
	a_message->	FindString("URL_TYPE", count, & url);
	
	url.ToLower();

	// find menu	
	if (url == SPRINGBOARD_FILE_URL_TYPE)		menu =	m_file_menu;
	if (url == SPRINGBOARD_FTP_URL_TYPE)		menu =	m_ftp_menu;
	if (url == SPRINGBOARD_HTTP_URL_TYPE)		menu =	m_http_menu;
	if (url == SPRINGBOARD_HTTPS_URL_TYPE)		menu =	m_https_menu;
	if (url == SPRINGBOARD_RTSP_URL_TYPE)		menu =	m_rtsp_menu;
	if (url == SPRINGBOARD_TELNET_URL_TYPE)		menu =	m_telnet_menu;
	if (url == SPRINGBOARD_MAILTO_URL_TYPE)		menu =	m_mailto_menu;

	// remove previous menu items
	if (menu->CountItems() > 0)
		while ((item = menu->RemoveItem((int32) 0)) != NULL)		delete item;

	// add menu items
	for (count = 0; count < subs; count++)
	{
		a_message->	FindString("applications", count, & app);
		app.ToLower();
				
		add_URL_app	(menu, & url, & app);
	}

}

void
FavoriteWindow::add_URL_app	(BMenu * a_menu, BString * a_url, BString * a_app)
{
	PRINT(("FavoriteWindow::add_URL_app()\n"));
	
	a_url->ToLower();
	a_app->ToLower();

	// create menu_item
	BMimeType	app_mime		(a_app->String());
	BMenuItem	*	item		=	new	BMenuItem(NULL, NULL, 0, 0);
	BMessage	*	msg			=	new BMessage(SPRINGBOARD_URL_APP_CHANGE);
	char		*	app_name	=	new	char [B_MIME_TYPE_LENGTH];	
	
	// label of app menuitem
	app_mime.GetShortDescription(app_name);		

	if (app_name == NULL)	item->	SetLabel(a_app->String());
	else					item->	SetLabel(app_name);

	// target & message of app menuitem
	item->	SetTarget(be_app);
	msg->	AddString("URL_TYPE", a_url->String());
	msg->	AddString("APP_SIG", a_app->String());
	item->	SetMessage(msg);
	
	// add menu_item to its URL-menu
	a_menu->	AddItem(item);

	delete	app_name;
}

void
FavoriteWindow::set_URL_pref_apps	(BMessage * a_message)
{
	PRINT(("FavoriteWindow::set_URL_pref_apps()\n"));
	
	BString			url;
	BString			app;
	
	a_message->	FindString(SPRINGBOARD_FILE_URL_TYPE, & app);
	set_URL_pref_app	(m_file_menu, & url, & app);
	
	a_message->	FindString(SPRINGBOARD_FTP_URL_TYPE, & app);
	set_URL_pref_app	(m_ftp_menu, & url, & app);
	
	a_message->	FindString(SPRINGBOARD_HTTP_URL_TYPE, & app);
	set_URL_pref_app	(m_http_menu, & url, & app);
	
	a_message->	FindString(SPRINGBOARD_HTTPS_URL_TYPE, & app);
	set_URL_pref_app	(m_https_menu, & url, & app);
	
	a_message->	FindString(SPRINGBOARD_RTSP_URL_TYPE, & app);
	set_URL_pref_app	(m_rtsp_menu, & url, & app);
	
	a_message->	FindString(SPRINGBOARD_TELNET_URL_TYPE, & app);
	set_URL_pref_app	(m_telnet_menu, & url, & app);
	
	a_message->	FindString(SPRINGBOARD_MAILTO_URL_TYPE, & app);
	set_URL_pref_app	(m_mailto_menu, & url, & app);
}

void
FavoriteWindow::set_URL_pref_app	(BMenu * a_menu, BString * a_url, BString * a_app)
{
	PRINT(("FavoriteWindow::set_URL_pref_app()\n"));
	
	a_url->ToLower();
	a_app->ToLower();

	BMenuItem	*	item		=	NULL;
	bool			got_item	=	false;

	BString		string;
	int32		count	=	0;
	
	while ((item = a_menu->ItemAt(count++)))
	{
		BMessage * msg	=	item->Message();
		
		msg->	FindString("APP_SIG", & string);
	
		if (*a_app == string)
		{
			got_item	=	true;
			item->SetMarked(true);
		}
	}

	if (! got_item)
		add_URL_app	(a_menu, a_url, a_app);
}
