#define DEBUG 0
#include <Debug.h>

#include <stdio.h>
#include <unistd.h>

#include <Roster.h>
#include <Alert.h>
#include <PopUpMenu.h>
#include <Menu.h>
#include <MenuItem.h>
#include <String.h>
#include <Path.h>
#include <Entry.h>
#include <FindDirectory.h>
#include <Directory.h>

#include <View.h>
#include <Deskbar.h>
#include <Bitmap.h>
#include <AppFileInfo.h>

#include "Defs.h"
#include "SpringboardReplicant.h"

const rgb_color COLOR_FOREGROUND = { 0, 0, 0 };
const rgb_color COLOR_BACKGROUND = { 255, 203, 102 };

SpringboardReplicant::SpringboardReplicant	(void)
	: BView(BRect(0, 0, 15, 15), 0,0,0)
{
	PRINT(("SpringboardReplicant::SpringboardReplicant()\n"));

}

SpringboardReplicant::~SpringboardReplicant	(void)	
{
	PRINT(("SpringboardReplicant::~SpringboardReplicant()\n"));
}

SpringboardReplicant::SpringboardReplicant(BMessage * a_message)
	: BView(BRect(0,0,15,15), SPRINGBOARD_REPLICANT, B_FOLLOW_NONE, B_WILL_DRAW) 
{
	PRINT(("SpringboardReplicant::SpringboardReplicant(BMessage * a_message)\n"));

	m_popup_menu	=	NULL;
	icon			=	NULL;

	CreatePopup();
	
	entry_ref	ref;
	if (be_roster->FindApp(SPRINGBOARD_APP_SIG, & ref) == B_OK)
	{
		BFile file	(& ref, O_RDONLY);
		BAppFileInfo fileInfo(& file);
		
		icon	=	new BBitmap	(Bounds(), B_COLOR_8_BIT, true);
		
		if (fileInfo.GetIcon(icon, B_MINI_ICON) != B_OK)
			icon = NULL;
	}
}

__declspec(dllexport) SpringboardReplicant * 
SpringboardReplicant::Instantiate	(BMessage * archive)
{
	PRINT(("__declspec(dllexport) SpringboardReplicant * "
			"SpringboardReplicant::Instantiate	(BMessage * archive)\n"));

	if (! validate_instantiation(archive, "SpringboardReplicant")) 
	{
		PRINT(("! validate_instantiation()\n"));
		return 0;
	}
	return new SpringboardReplicant(archive);
}

status_t 
SpringboardReplicant::Archive(BMessage * archive, bool deep) const
{
	PRINT(("status_t SpringboardReplicant::Archive(BMessage * archive, bool deep) const\n"));

	// Tell the overridden BView to archive itself.
	BView::Archive(archive, deep);
  
	// Store our signature and the name of our class in the archive.
	archive->AddString("add_on", SPRINGBOARD_APP_SIG);
	archive->AddString("class", "SpringboardReplicant");

	return B_OK;
}

void 
SpringboardReplicant::AttachedToWindow ()
{
	SetViewColor(Parent()->ViewColor());
}

void 
SpringboardReplicant::Draw(BRect updateRect)
{
	BView::Draw(Bounds());

	SetDrawingMode(B_OP_OVER);
	
	if (icon != NULL)
		DrawBitmap(icon);
	else
	{
		// Grey fill
		SetHighColor(Parent()->ViewColor());
		FillRect(updateRect);

		// Black outline 
		SetHighColor(COLOR_FOREGROUND);
		StrokeRect(Bounds());
	}
}

void
SpringboardReplicant::MouseDown(BPoint point) 
{
	ConvertToScreen(&point); 
	m_popup_menu->SetTargetForItems(this);
	m_popup_menu->Go(point, true, false, false); 
}

void
SpringboardReplicant::MessageReceived	(BMessage * a_message)
{
	switch(a_message->what) 
	{
		case SPRINGBOARD_LAUNCH_URL_FROM_CLIPBOARD: 
			LaunchClipboard();
			break;
			
		case SPRINGBOARD_BOOKMARK_URL_FROM_CLIPBOARD: 
			BookmarkClipboard();
			break;
		
		case SPRINGBOARD_OPEN_BOOKMARK_FOLDER: 
			OpenBookmarkFolder();
			break;
			
		case SPRINGBOARD_OPEN_SPRINGBOARD: 
			be_roster->Launch(SPRINGBOARD_APP_SIG);
			break;
			
		case B_QUIT_REQUESTED:
		{
			//BMessage	msg	(SPRINGBOARD_REPLICANT_QUITS);
			BMessenger	mssgr	(SPRINGBOARD_APP_SIG);
			if (mssgr.IsValid())
				mssgr.SendMessage(SPRINGBOARD_REPLICANT_QUITS);
			
			//be_roster->Launch(SPRINGBOARD_APP_SIG, & msg);
			
			BDeskbar deskbar;
			deskbar.RemoveItem(SPRINGBOARD_REPLICANT);   // bad ?
			break;
		}
		
		default: 
		{
			BView::MessageReceived(a_message);
			break;
		}
	}
}

void
SpringboardReplicant::CreatePopup	(void) 
{
	m_popup_menu = new BPopUpMenu("TuneTrackerPopup", false, false);
	BMenuItem * item; 

	item = new BMenuItem("Launch URL from Clipboard" B_UTF8_ELLIPSIS, 
							new BMessage(SPRINGBOARD_LAUNCH_URL_FROM_CLIPBOARD));
	m_popup_menu->AddItem(item);
	
	item = new BMenuItem("Bookmark URL from Clipboard", 
							new BMessage(SPRINGBOARD_BOOKMARK_URL_FROM_CLIPBOARD));
	m_popup_menu->AddItem(item);
	
	item = new BMenuItem("Open Bookmark Folder" B_UTF8_ELLIPSIS, 
							new BMessage(SPRINGBOARD_OPEN_BOOKMARK_FOLDER));
	m_popup_menu->AddItem(item);
	
	item = new BMenuItem("Open Springboard" B_UTF8_ELLIPSIS, 
							new BMessage(SPRINGBOARD_OPEN_SPRINGBOARD));
	m_popup_menu->AddItem(item);
	
	m_popup_menu->AddSeparatorItem();

	item = new BMenuItem("Quit", new BMessage(B_QUIT_REQUESTED));
	m_popup_menu->AddItem(item);

	m_popup_menu->SetTargetForItems(this);
}

void
SpringboardReplicant::LaunchClipboard		(void) 
{
	BMessage msg(B_ARGV_RECEIVED); 
	msg.AddString("argv", "Springboard"); 
	msg.AddString("argv", "--clipboard");  
	msg.AddInt32("argc", 2); 
	be_roster->Launch (SPRINGBOARD_APP_SIG, &msg ); 
}

void
SpringboardReplicant::BookmarkClipboard		(void) 
{
	BMessage msg(B_ARGV_RECEIVED); 
	msg.AddString("argv", "Springboard"); 
	msg.AddString("argv", "--clip2bookmark");  
	msg.AddInt32("argc", 2); 
	be_roster->Launch (SPRINGBOARD_APP_SIG, &msg ); 
}

void
SpringboardReplicant::OpenBookmarkFolder	(void) 
{
	system("/system/Tracker /boot/home/config/settings/NetPositive/Bookmarks &");
}
