// URL_file.cpp

#define DEBUG 0
#include <Debug.h>

#include <stdio.h>

#include <File.h>
#include <Volume.h>
#include <NodeInfo.h>
#include <String.h>
#include <fs_attr.h>

#include "Defs.h"
#include "URL_file.h"

URL_file::URL_file(const entry_ref * a_ref)	:	BFile(a_ref, B_READ_WRITE)
{
	m_ref = *a_ref;

	char * type	=	new char [B_MIME_TYPE_LENGTH];
	BNodeInfo	node_inf	(this);
	node_inf.GetType(type);
	m_type	=	type;
	m_type.ToLower();
	delete	type;

	if (m_type == SPRINGBOARD_BOOKMARK_TYPE)
		InitBookmark();
	else if (m_type == SPRINGBOARD_FAVORITE_TYPE)
		InitFavorite();
	else	// favorite on non-BFS volume
	{								
		BString	name(m_ref.name);
		name.ToLower();
		if(name.FindLast(".url") >= 0)
			InitFavorite();
	}
}

URL_file::~URL_file()	{	}

void 
URL_file::InitFavorite()
{
	PRINT(("InitFavorite()\n"));

	// Title	
	m_title = m_ref.name;
	m_title.RemoveLast(".url");
	m_title.RemoveLast(".URL");

	// URL
	off_t		size;
	GetSize(&size);

	if (size <= 0)
		return;

	char * buffer	=	new	char [size+1]; 
	Read((void *)buffer, size);
	buffer[size] = '\0';	
	m_url = buffer;
	delete buffer;
		
	int32 thumb;
	thumb	=		m_url.FindFirst("[InternetShortcut]");
	m_url.Remove(0, thumb + 19);
			
	thumb		=		m_url.FindFirst("=");
	m_url.Remove(0, thumb+1);

	thumb		=		m_url.FindFirst("\n");
	m_url.Truncate(thumb-1);
}

void 
URL_file::InitBookmark()
{
	PRINT(("InitBookmark()\n"));

	attr_info	info;
	ssize_t		bytes;
	
	GetAttrInfo("META:url", & info);
	char * buffer = new char [info.size + 1];
	bytes = ReadAttr("META:url", B_STRING_TYPE, 0, buffer, info.size);
	buffer[info.size]	=	'\0';
	if (bytes > 0)
		m_url	=	buffer;
	delete buffer;
	
	//PRINT(("META:url: %s\n", m_url.String()));
	
	GetAttrInfo("META:title", & info);
	buffer = new char [info.size + 1];
	bytes = ReadAttr("META:title", B_STRING_TYPE, 0, buffer, info.size);
	buffer[info.size]	=	'\0';
	if (bytes > 0)
		m_title	=	buffer;
	delete buffer;

	//PRINT(("META:title: %s\n", m_title.String()));
}

status_t 
URL_file::Convert()
{
	PRINT(("URL_file::Convert()\n"));

	BVolume		vol	(m_ref.device);
	if (! vol.KnowsMime() || ! vol.KnowsAttr())
		return B_ERROR;  // aborts batch of conversions

	if (m_type == SPRINGBOARD_FAVORITE_TYPE)
		FavToBookmark();
	if (m_type == SPRINGBOARD_BOOKMARK_TYPE)
		BookToFavorite();
	
	return B_OK;
}

void 
URL_file::FavToBookmark		(void)
{
	if (m_url.Length() <= 0 || m_title.Length() <= 0)
		return;
		
	ssize_t		written_bytes = 0;
	
	// add META:url attribute
    written_bytes = WriteAttr	("META:url", B_STRING_TYPE, 0,
										m_url.String(), m_url.Length()+1);

	if (written_bytes != m_url.Length()+1)
		return;
		
	// add META:title attribute
    written_bytes = WriteAttr	("META:title", B_STRING_TYPE, 0,
										m_title.String(), m_title.Length()+1);

	if (written_bytes != m_title.Length()+1)
		return;
		
	// add BEOS:TYPE attribute
	m_type 	=	"application/x-vnd.Be-bookmark";

    written_bytes = WriteAttr	("BEOS:TYPE", B_MIME_TYPE, 0, 
    									 m_type.String(), m_type.Length()+1);

	if (written_bytes != m_type.Length()+1)
		return;

	// remove file contents
	SetSize(0);
	
	// rename file (remove ".url")
	BEntry	entry	(& m_ref);
	BString	name	(m_ref.name);
	name.ToLower();
	if (name.FindLast(".url") == name.Length() -4)
	{
		name	=	m_ref.name;
		name.Remove(name.Length() -4, 4);
	}
	else
		name	=	m_ref.name;
	if (entry.Rename(name.String()) != B_OK)
		return;
	
}

void 
URL_file::BookToFavorite	(void)
{
	// add contents to file
	// [InternetShortcut] + URL -> BString
	// write to file
	
	BString contents	("[InternetShortcut]\r\nURL=");
	contents	+=	m_url;
	contents	+=	"\r\n";

	if (! (Write(contents.String(), contents.Length()) > 0))
		return;
	
	// rename file (add .url)
	BEntry	entry	(& m_ref);
	BString	name	(m_ref.name);
	name		+=	".url";
	if (entry.Rename(name.String()) != B_OK)
		return;
	
	// change filetype
	BNodeInfo	info	(this);
	if (info.SetType(SPRINGBOARD_FAVORITE_TYPE) != B_OK)
		return;
	
	// remove attributes
	
	RemoveAttr("META:url");
	RemoveAttr("META:title");
	//RemoveAttr("META:keyw");
	RemoveAttr("BEOS:PREF_APP");
}

const char * 
URL_file::Title	(void)
{
	return m_title.String();	
}

const char * 
URL_file::URL	(void)
{
	return m_url.String();
}

status_t
URL_file::InitCheck	(void)
{
	if (m_url.Length() > 0)
		return B_OK;
	else
		return B_ERROR;
}
